Attribute VB_Name = "General"
Option Explicit

' Global Variables
Public gMaxAvge As Integer               ' Maximum rider average
Public gDecSep As String                 ' Locale decimal separator
Public gDateFmt As String                ' Locale date format
Public gFindText As String               ' Search string
Public gMatchWord As Integer             ' Determines whether case sensitivity applies when searching
Public gMatchCase As Integer             ' Determines whether to match only whole words when searching

' Variables used in API Calls
Global lpPrevWndProc As Long

Private pMinX As Long
Private pMinY As Long
Private pMaxX As Long
Private pMaxY As Long

Private Type POINTAPI
    x As Long
    y As Long
End Type

Private Type MINMAXINFO
    ptReserved As POINTAPI
    ptMaxSize As POINTAPI
    ptMaxPosition As POINTAPI
    ptMinTrackSize As POINTAPI
    ptMaxTrackSize As POINTAPI
End Type

' Global Constants
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_USERS = &H80000003
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const REG_SZ = 1
Public Const ERROR_SUCCESS = 0&

Public Const LOCALE_SDECIMAL As Long = &HE
Public Const LOCALE_SSHORTDATE As Long = &H1F

Public Const GWL_WNDPROC As Long = -4
Public Const WM_GETMINMAXINFO As Long = &H24

' API Declarations
Declare Sub CopyMemoryToMinMaxInfo Lib "KERNEL32" Alias "RtlMoveMemory" (hpvDest As MINMAXINFO, ByVal hpvSource As Long, ByVal cbCopy As Long)
Declare Sub CopyMemoryFromMinMaxInfo Lib "KERNEL32" Alias "RtlMoveMemory" (ByVal hpvDest As Long, hpvSource As MINMAXINFO, ByVal cbCopy As Long)

Declare Function RegCloseKey Lib "advapi32.dll" (ByVal HKEY As Long) As Long
Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal HKEY As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Declare Function RegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" (ByVal HKEY As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal HKEY As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" (ByVal HKEY As Long, ByVal lpSubKey As String, ByVal lpValue As String, lpcbValue As Long) As Long
Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal HKEY As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal HKEY As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal HKEY As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, lpData As Any, ByVal cbData As Long) As Long

Declare Function DefWindowProc Lib "user32" Alias "DefWindowProcA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Declare Function GetUserDefaultLCID Lib "KERNEL32" () As Long
Declare Function GetLocaleInfo Lib "KERNEL32" Alias "GetLocaleInfoA" (ByVal Locale As Long, ByVal LCType As Long, ByVal lpLCData As String, ByVal cchData As Long) As Long

Public Sub Main()

    ' Creates or updates file associations
    SaveRegString HKEY_CLASSES_ROOT, "SpwyMtg.RiderDB", "", "Rider Database"
    SaveRegString HKEY_CLASSES_ROOT, ".rdb", "", "SpwyMtg.RiderDB"
    SaveRegString HKEY_CLASSES_ROOT, "SpwyMtg.RiderDB\shell\open\command", "", App.Path + "\RdrEditor.exe" + " %1"
    SaveRegString HKEY_CLASSES_ROOT, "SpwyMtg.RiderDB\DefaultIcon", "", App.Path + "\RdrEditor.exe" + ",0"
    
    ' Gets maximum average
    gMaxAvge = Val(GetRegString(HKEY_CURRENT_USER, "Software\Speedway Meeting", "DefMaxAvge"))
    
    ' Checks maximum average is valid
    Select Case gMaxAvge
        Case Is < 1, Is > 99
            gMaxAvge = 12
    End Select
    
    ' Gets locale-specific information
    gDecSep = GetUserLocaleInfo(LOCALE_SDECIMAL)
    
    Load frmRdrEditor

End Sub

Public Function GetRegString(HKEY As Long, KeyPath As String, KeyName As String) As String
       
    Dim KeyHand As Long                     ' Registry key handle
    Dim Result As Long                      ' Returned error code
    Dim ValueType As Long                   ' Key type
    Dim Buffer As String                    ' Key value
    Dim DataBufferSize As Long              ' Length of key value
    Dim ZeroPos As Integer                  ' Check variable
    
    ' Reads value of Registry key
    RegOpenKey HKEY, KeyPath, KeyHand
        
    Result = RegQueryValueEx(ByVal KeyHand, ByVal KeyName, 0&, ValueType, ByVal 0&, DataBufferSize)
    
    If Result = ERROR_SUCCESS And ValueType = REG_SZ Then
        
        Buffer = String(DataBufferSize, " ")
        Result = RegQueryValueEx(KeyHand, KeyName, 0&, 0&, ByVal Buffer, DataBufferSize)
            
        If Result = ERROR_SUCCESS Then
            
            ZeroPos = InStr(Buffer, Chr(0))
            
            If ZeroPos > 0 Then
                GetRegString = Left(Buffer, ZeroPos - 1)
            Else
                GetRegString = Buffer
            End If
        
        End If
    
    End If
    
    RegCloseKey KeyHand

End Function

Public Function GetUserLocaleInfo(ByVal LocaleType As Long) As String

    Dim LocaleID As Long
    Dim Returned As String
    Dim Result As Long

    LocaleID = GetUserDefaultLCID()
    Result = GetLocaleInfo(LocaleID, LocaleType, Returned, Len(Returned))
    
    If Result > 0 Then
    
      Returned = Space(Result)
      Result = GetLocaleInfo(LocaleID, LocaleType, Returned, Len(Returned))

      If Result > 0 Then GetUserLocaleInfo = Left(Returned, Result - 1)
   
   End If
    
End Function

Public Sub SetFormLimits(hWnd As Long, MinX As Integer, MinY As Integer, MaxX As Integer, MaxY As Integer)
  
    pMinX = MinX / Screen.TwipsPerPixelX
    pMinY = MinY / Screen.TwipsPerPixelY
    pMaxX = MaxX / Screen.TwipsPerPixelX
    pMaxY = MaxY / Screen.TwipsPerPixelY
  
    lpPrevWndProc = SetWindowLong(hWnd, GWL_WNDPROC, AddressOf WindowProc)

End Sub

Public Sub UnsetFormLimits(hWnd As Long)
    
    SetWindowLong hWnd, GWL_WNDPROC, lpPrevWndProc

End Sub

Public Function WindowProc(ByVal hw As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

    Dim MinMax As MINMAXINFO

    If uMsg = WM_GETMINMAXINFO Then
        CopyMemoryToMinMaxInfo MinMax, lParam, Len(MinMax)
        MinMax.ptMinTrackSize.x = pMinX
        MinMax.ptMinTrackSize.y = pMinY
        MinMax.ptMaxTrackSize.x = pMaxX
        MinMax.ptMaxTrackSize.y = pMaxY
        CopyMemoryFromMinMaxInfo lParam, MinMax, Len(MinMax)
        WindowProc = DefWindowProc(hw, uMsg, wParam, lParam)
    Else
        WindowProc = CallWindowProc(lpPrevWndProc, hw, uMsg, wParam, lParam)
    End If

End Function

Public Sub Parse(Source As String, Separator As String, Field() As String, NumFields As Integer)
  
    Dim FieldID As Integer               ' Identifies parsed fields
    ReDim Field(NumFields)
  
    Do While Len(Source) > 0 And FieldID <> NumFields
        FieldID = FieldID + 1
        Source = CStr(Source)
        If InStr(1, Source, Separator) = 0 Then
            Field(FieldID) = Trim(Source)
            Source = ""
        Else
            Field(FieldID) = Trim(Left(Source, InStr(1, Source, Separator) - 1))
            Source = Mid(Source, InStr(1, Source, Separator) + Len(Separator))
        End If
    Loop
  
End Sub

Public Sub SaveRegString(HKEY As Long, KeyPath As String, KeyName As String, KeyValue As String)

    Dim KeyHand As Long                     ' Key handle
    
    ' Saves Registry key
    RegCreateKey HKEY, KeyPath, KeyHand
    RegSetValueEx KeyHand, KeyName, 0, REG_SZ, ByVal KeyValue, Len(KeyValue)
    RegCloseKey KeyHand

End Sub
